# generates AUTO files from model definitions in *.cfg files
import auto, os, inspect, scipy, sys

# There was a problem if you install the pylib folder into a different
# path. The c.fp and c.bvp files would not be found. Now we determine
# the module path (by assuming that constant files are in the same directory)
# hopefully this is robust enough!
modulepath = (sys.modules[__name__].__file__).rsplit("/",1)[0] + "/"

def var2unames(var):
  """ takes list of variable names """
  from scipy import arange
  return dict(zip(arange(len(var),dtype=int)+1,map(str,var)))

def par2pnames(bifpar):
  """ takes list of parameter names """
  from scipy import arange
  if 'period' in bifpar: bifpar.remove('period')
  p2n= dict(zip(arange(len(bifpar),dtype=int)+1,map(str,bifpar)))
  if p2n.has_key(11):
    p2n[max(p2n.keys())+1]= p2n.pop(11)
  p2n[11]= 'period'
  return p2n

def var2advar(var):
  return ['ad'+v for v in var]

#def ode2adjointsystem(rhs):

def writeCheader(headername,unames={},pnames={}):
  """ Write header in C format.
      Requires unames and pnames dictionaries as used by AUTO.
  """
  from string import join
  unames=dict([(x-1,y) for x,y in unames.items()])
  pnames=dict([(x-1,y) for x,y in pnames.items()])
  if not headername[-2:] == '.h':
    headername+='.h'
  f=open(headername,'w')
  f.write( join(map(lambda x: '#define {1}\tu[{0}]'.format(*x),unames.items())\
              + map(lambda x: '#define {1}\tpar[{0}]'.format(*x),pnames.items())\
              + map(lambda x: '#define {1}_left\tu0[{0}]'.format(*x),unames.items())\
              + map(lambda x: '#define {1}_right\tu1[{0}]'.format(*x),unames.items()),'\n') )

def writeBVP(fname,**kwargs):
  """ Generates fname.h, fname.c, c.fname
      requires:
        rhs: list of the right hand side of the ODE system
        var: dict of state variables (with initial values)
        bifpar: dict of continuation parameters (with initial values)
        bc: list of boundary conditions
        ic: list of integral conditions
  """
  from string import join
  from parseC import parseC
  bvp=True # HACK
  if not 'bifpar' in kwargs:
    raise Exception('No bifpar argument found.')
  if not 'var' in kwargs:
    raise Exception('No var argument found.')
  if not 'rhs' in kwargs:
    raise Exception('No rhs argument found.')
  if not 'bc' in kwargs:
    raise Exception('No bc argument found.')
  if not 'ic' in kwargs:
    raise Exception('No ic argument found.')
  bifpar= kwargs['bifpar']
  rhs= kwargs['rhs']
  var= kwargs['var']
  bc= kwargs['bc']
  ic= kwargs['ic']
  try:
    pnames= par2pnames(bifpar.keys()) # pnames {1:'parname1',2:'parname2'}
    reverse_pnames= dict([(v,k) for k,v in pnames.items()])
    initpnames= dict([(i,0) for i in pnames])
    initpnames.update(dict([(reverse_pnames[v],bifpar[v]) for v in bifpar]))
  except AttributeError:
    pnames= par2pnames(bifpar)
    initpnames= dict([(i,0) for i in pnames])
  try:
    unames= var2unames(var.keys())
    reverse_unames= dict([(v,k) for k,v in unames.items()])
    initunames= dict([(i,0) for i in unames])
    initunames.update(dict([(reverse_unames[v],var[v]) for v in var]))
  except AttributeError:
    unames= var2unames(var)
    initunames= dict([(i,0) for i in unames])
  writeCheader(fname,unames=unames,pnames=pnames) # write fname.h file
  # write to files
  s= '#include <stdio.h>\n#include <math.h>\n#include "auto_f2c.h"\n'
  s+= '#include "{0}"\n\n'.format(fname+'.h')
  s+= join([list2cfunc(list2cf(rhs),bvp=True),\
            list2cstpnt(initpar=initpnames,initvar=initunames),\
            list2cicnd(ic),\
            list2cbcnd(bc),\
            list2cfopt([]),\
            list2cplvs([])],'\n')
  f= open(fname+'.c','w')
  f.write(s)
  f.close()
  d= parseC(modulepath+'c.bvp')
  #d= parseC(os.path.expanduser('~/lib/pylib/c.bvp'))
  d['NINT']= len(ic)
  d['NBC']= len(bc)
  d['ICP']= bifpar
  d['NDIM']= len(rhs)
  d['ICP']= bifpar
  print 'For boundary value problems AUTO requires:'
  print '{0}>={4}={1}+{2}-{3}+1 (ICP>=NBC+NINT-NDIM+1)\n'.format(len(d['ICP']),\
                  d['NBC'],d['NINT'],d['NDIM'],d['NBC']+d['NINT']-d['NDIM']+1)
  f= open('c.'+fname,'w')
  f.write(str(d))
  f.close()
  return unames, pnames


def writeFP(fname,**kwargs):
  """ Generates fname.h, fname.c, c.fname
      requires:
        rhs: list of the right hand side of the ODE system
        var: dict of state variables (with initial values)
        bifpar: dict of continuation parameters (with initial values)
        bc: list of boundary conditions
        ic: list of integral conditions
  """
  from string import join
  from parseC import parseC
  if not 'bifpar' in kwargs:
    raise Exception('No bifpar argument found.')
  if not 'var' in kwargs:
    raise Exception('No var argument found.')
  if not 'rhs' in kwargs:
    raise Exception('No rhs argument found.')
  if not 'bc' in kwargs:
    raise Exception('No bc argument found.')
  if not 'ic' in kwargs:
    raise Exception('No ic argument found.')
  bifpar= kwargs['bifpar']
  rhs= kwargs['rhs']
  var= kwargs['var']
  bc= kwargs['bc']
  ic= kwargs['ic']
  try:
    pnames= par2pnames(bifpar.keys()) # pnames {1:'parname1',2:'parname2'}
    reverse_pnames= dict([(v,k) for k,v in pnames.items()])
    initpnames= dict([(i,0) for i in pnames])
    initpnames.update(dict([(reverse_pnames[v],bifpar[v]) for v in bifpar]))
  except AttributeError:
    pnames= par2pnames(bifpar)
    initpnames= dict([(i,0) for i in pnames])
  try:
    unames= var2unames(var.keys())
    reverse_unames= dict([(v,k) for k,v in unames.items()])
    initunames= dict([(i,0) for i in unames])
    initunames.update(dict([(reverse_unames[v],var[v]) for v in var]))
  except AttributeError:
    unames= var2unames(var)
    initunames= dict([(i,0) for i in unames])
  writeCheader(fname,unames=unames,pnames=pnames) # write fname.h file
  # write to files
  s= '#include <stdio.h>\n#include <math.h>\n#include "auto_f2c.h"\n'
  s+= '#include "{0}"\n\n'.format(fname+'.h')
  s+= join([list2cfunc(list2cf(rhs),bvp=False),\
            list2cstpnt(initpar=initpnames,initvar=initunames),\
            list2cicnd(ic),\
            list2cbcnd(bc),\
            list2cfopt([]),\
            list2cplvs([])],'\n')
  f= open(fname+'.c','w')
  f.write(s)
  f.close()
  d= parseC(modulepath+'c.fp')
  #d= parseC(os.path.expanduser('~/lib/pylib/c.fp'))
  d['NINT']= len(ic)
  d['NBC']= len(bc)
  d['ICP']= bifpar
  d['NDIM']= len(rhs)
  d['ICP']= bifpar
  print 'For fix point continuation AUTO requires:'
  print '{0}={1}+{2}-{3}+1 (ICP=NBC+NINT-NDIM+1)\n'.format(len(d['ICP']),d['NBC'],d['NINT'],d['NDIM'])
  f= open('c.'+fname,'w')
  f.write(str(d))
  f.close()
  return unames, pnames


def rollsolution(s,ix):
    """ s : AUTO Solution object
        ix : index which will be t=0
    """
    from scipy import ones, mod, roll, r_
    orb= ones((len(s.coordnames)+1,len(s['t'])))
    orb[:,:-1]= r_[scipy.mod(roll(s['t'][:-1],-ix)-s['t'][ix],1)[None,:],roll(s.coordarray[:,:-1],-ix,1)]
    orb[1:,-1]= orb[1:,0]
    return orb

def initadjoint(s, f, v, p, dotZF=.5, NTST=300, Nperiods=4, umaxname=None):
    """ s : AUTO solution object
        f : r.h.s. of the ODE
        v : variable
        p : parameters and functions
    """
    import scipy, sympy
    from scipy.interpolate import interp1d
    from scipy.integrate import odeint
    from sympy import S, lambdify
    if not umaxname: umaxname=s.coordnames[0]
    orb= rollsolution(s,s[umaxname].argmax())
    orbbar= interp1d(orb[0,:],orb[1:,:])
    orbfoo= lambda tt: orbbar(scipy.mod(tt,1))
    J=[[S(fk).subs(p).subs(p).diff(vk).doit() for vk in v] for fk in f]
    Jfoo= lambdify(v, J)
    adsys= lambda x,tt:scipy.dot(x,Jfoo(*orbfoo(-tt)))*s['period']
    tt= scipy.linspace(0,Nperiods,Nperiods*NTST)
    ady= odeint(adsys,scipy.randn(len(v)),tt)[::-1,:][:NTST+1,:]
    tmrhs= lambdify(v,[S(fk).subs(p).subs(p) for fk in f])
    goldstone= scipy.apply_along_axis(lambda x: tmrhs(*x),0,orb[1:,:]) #/ sBVP['period'] # period wrong?
    adorbfoo= interp1d(tt[:NTST+1],ady.T)
    dat= adorbfoo(orb[0,:])
    dotZF_old= scipy.trapz((dat * goldstone).sum(0),orb[0,:])
    dat/= dotZF_old / dotZF
    return dat


##### FORTRAN CODE GEN #####

def writeHOM(fname,ode,bifpar,bc=[],ic=[],fopt=[],initvar=[]):
  """ CAREFUL: Slitly different arguments then the other write routines!
      WARNING: Do not use parameter and variable names that are used by 
               FORTRAN, e.g., I, U, T, PAR.
  """
  from sympy.printing import fcode
  from sympy import S
  ode = [(fcode(S(i)),fcode(S(j))) for i,j in ode]
  var, rhs = zip(*ode)
  bc = [fcode(S(k)) for k in bc]
  ic = [fcode(S(k)) for k in ic]
  pn = [(i+1,j) for i,(j,k) in enumerate(bifpar)]
  un = [(i+1,j) for i,j in enumerate(var)]
  init = bifpar + initvar
  f=open(fname,'w')
  f.write( "\n\n".join([ list2ffunc(ode,un,pn),
    list2fstpnt(init, un, pn), 
    list2pvls(), list2bcnd(bc), list2icnd(ic) ])
    )
  f.close()
  return un, pn

# Fortrancode!!!!

def list2ffunc(rhs,un,pn,bvp=False):
  """ paste a string or list of strings into the f90 FUNC"""
  from sympy import S
  from sympy.printing import ccode
  if bvp:
    mulper= '  int i;\n  for (i = 0; i < {0}; i++)\n    f[i]*= period;'.format(str(len(rhs)))
  else:
    mulper= ""
  rhs= [" = ".join(k) for k in rhs]
  rhs= "\n  ".join(rhs)
  ptalias = ["DOUBLE PRECISION, POINTER :: {}".format(j) for i,j in un]\
      + ["DOUBLE PRECISION, POINTER :: {}".format(j) for i,j in pn]\
      + ["{} => U({})".format(j,i) for i,j in un]\
      + ["{} => PAR({})".format(j,i) for i,j in pn]
  ptalias= "\n  ".join(ptalias)
  funcstr = """
  SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP) 
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM,ICP(*),IJAC
  DOUBLE PRECISION, INTENT(IN), TARGET :: U(NDIM),PAR(*)
  DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
  DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,NDIM),DFDP(NDIM,*)

  ! POINTER ALIASES TO U AND PAR !
  {1}

  ! RHS !
  {0}

  END SUBROUTINE FUNC
  """
  return funcstr.format(rhs,ptalias)


def list2fstpnt(init, un, pn):
  """ WARNING: Do not use parameters that AUTO uses. In particular
      param T can not be used, since AUTO uses it.
      These parameter conflicts could be resolved if no aliases
      were used in the code. However, this would reduce readability 
      of the code.
  """
  assert "T" not in [i for i,j in init]
  initstr = ["{} = {}".format(i,j) for i,j in init]
  initstr = "\n  ".join(initstr)
  ptalias = ["DOUBLE PRECISION, POINTER :: {}".format(j) for i,j in un]\
      + ["DOUBLE PRECISION, POINTER :: {}".format(j) for i,j in pn]\
      + ["{} => U({})".format(j,i) for i,j in un]\
      + ["{} => PAR({})".format(j,i) for i,j in pn]
  ptalias= "\n  ".join(ptalias)
  stpntstr = """
  SUBROUTINE STPNT(NDIM,U,PAR,T) 
  ! Sets parameter values for homoclinic bifurcation analysis (IPS=9).  
  
  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM),PAR(*)
  DOUBLE PRECISION, INTENT(IN) :: T

  ! POINTER ALIASES TO U AND PAR !
  {1}
  
  ! COMMON block needed if IPS=9 (homoclinic bifurcations) :
  INTEGER ITWIST,ISTART,IEQUIB,NFIXED,NPSI,NUNSTAB,NSTAB,NREV
  COMMON /BLHOM/ ITWIST,ISTART,IEQUIB,NFIXED,NPSI,NUNSTAB,NSTAB,NREV

  !----------------------------------------------------------------------
  ! Problem parameters (only PAR(1-9) are available to the user) :

  {0}

  !----------------------------------------------------------------------
  ! If IEQUIB=1 then put the equilibrium in PAR(11+i), i=1,...,NDIM :

  IF (IEQUIB.NE.0) THEN
  PAR(12) = 0.0
  PAR(13) = 0.0
  PAR(14) = 0.0
  ENDIF
  !----------------------------------------------------------------------
  ! Distance along the unstable manifold :
  IF (ISTART.EQ.3) THEN
  PAR(12+NDIM*IEQUIB)=-0.00001
  ENDIF
  !----------------------------------------------------------------------

  END SUBROUTINE STPNT
  """
  return stpntstr.format(initstr, ptalias)


def list2pvls():
  """ writ the equilibrium into par """
  pvlsstr = """
  SUBROUTINE PVLS(NDIM,U,PAR)

  IMPLICIT NONE
  INTEGER, INTENT(IN) :: NDIM
  DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
  DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)
  INTEGER I

  ! If IEQUIB=0 put analytic equilibrium in PAR(11+i), i=1,...,NDIM :

  ! DO I=1,NDIM
  !   PAR(11+i)=0
  ! ENDDO
  END SUBROUTINE PVLS
  """
  return pvlsstr


def list2bcnd(bc):
  bcndstr = """ 
  SUBROUTINE BCND
  END SUBROUTINE BCND
  """
  return bcndstr


def list2icnd(ic):
  icndstr = """
  SUBROUTINE ICND
  END SUBROUTINE ICND
  """
  return icndstr


def list2fopt(fopt):
  foptstr = """
  SUBROUTINE FOPT
  END SUBROUTINE FOPT
  """
  return foptstr


### END FORTRAN CODE GEN ###

###################### OLD ###########################


def cfg2autoFPLC(cfgname,bifpar=[],bvp=False,linsys=False,bc=[],lang='C',phasecond=''):
  """ Converts the model specified in cfgname into AUTO equation 
      and constants file. 
  """
  from neurosimlib import odeLC
  from sympy import S
  from misc import fullsubs, rmap
  from scipy import dot, transpose, r_, arange, zeros, eye
  from string import join
  from parseC import parseC
  import re
  ode= odeLC(cfgname)
  var= ode.var
  initbifpar=map(lambda x: ode.par.pop(x,0),bifpar) # exculde bifpar from substitution
  rhs= rmap(lambda x: fullsubs(x,dict(ode.par.items()+ode.fun.items())), S(ode.vfield))
  ics= []
  u2n= dict(zip(arange(len(var))+1,map(str,var)))
  n2u= dict(map(lambda x:x[::-1],u2n.items()))
  #if bc=='periodic':
  #  bc= [v+'0-'+v+'1' for v in map(str,var)]
  #if phasecond:
  #  phaseconstr= str(rhs[n2u[phasecond]-1])
  #  for s in map(str,var):
  #    phaseconstr= re.sub(r'\b{0}\b'.format(s),'{0}0'.format(s),phaseconstr)
  #  bc.append(phaseconstr)
  p2n= dict(zip(arange(len(bifpar))+1,map(str,bifpar)))
  if p2n.has_key(11):
    p2n[max(p2n.keys())+1]= p2n.pop(11)
  p2n[11]= 'period'
  bifpar.append('period')
  n2p= dict(map(lambda x:x[::-1],p2n.items()))
  # write to files
  headername= cfgname.replace('.cfg','FPLC.h')
  initpardic= dict(zip(p2n.keys(),zeros(len(p2n))))
  initpardic.update(dict(zip(map(lambda x: n2p[x],bifpar),initbifpar)))
  s= '#include <stdio.h>\n#include <math.h>\n#include "auto_f2c.h"\n'
  s+= '#include "{0}"\n\n'.format(headername)
  s+= join([list2cfunc(list2cf(rhs),bvp=bvp),list2cstpnt(initpar=initpardic),list2cicnd(ics),list2cbcnd(bc),list2cfopt([]),list2cplvs([])],'\n')
  f= open(headername.replace('.h','.c'),'w')
  f.write(s)
  f.close()
  f= open(headername,'w')
  f.write(list2cheader(u2n,p2n))
  f.close()
  d= parseC(os.path.expanduser('~/lib/pylib/c.fp'))
  d['NINT']= len(ics)
  d['NBC']= len(bc)
  d['ICP']= bifpar
  d['NDIM']= len(rhs)
  d['ICP']= bifpar
  print 'For fix point continuation AUTO requires:'
  print '{0}={1}+{2}-{3}+1 (ICP=NBC+NINT-NDIM+1)\n'.format(len(d['ICP']),d['NBC'],d['NINT'],d['NDIM'])
  f= open('c.'+headername.replace('.h',''),'w')
  f.write(str(d))
  f.close()
  return u2n, p2n


### OLD ###

def cfg2autoBVP(cfgname,bifpar=[],bvp=True,linsys=True,bc='periodic',lang='C',phasecond=''):
  """ Converts the model specified in cfgname into AUTO equation 
      and constants file.
      TODO: Get rid of the overhead of creating an ODE object (partly outdated,
      and no simulations are done anyway - OR SHOULD WE?).
  """
  from neurosimlib import odeLC
  from sympy import S
  from misc import fullsubs, rmap
  from scipy import dot, transpose, r_, arange, zeros, eye, array, iterable
  from string import join
  from parseC import parseC
  import re
  ode= odeLC(cfgname)
  var= ode.var
  initbifpar=map(lambda x: ode.par.pop(x,0),bifpar) # exculde bifpar from substitution
  rhs= rmap(lambda x: fullsubs(x,dict(ode.par.items()+ode.fun.items())), S(ode.vfield))
  ics= []
  if linsys:
    floqexp='lambda0'
    J= (ode.getJac() - S(floqexp)*eye(len(var))).tolist()
    alinvar= S(map(lambda x: 'ad'+x,ode.var))
    if type(linsys)==type(slice(0)):
      J= (array(J)[linsys,linsys]).tolist()
      alinvar= alinvar[linsys]
    Jmat= rmap(lambda x: fullsubs(x,dict(ode.par.items()+ode.fun.items())), S(J))
    rhs+= dot(-transpose(Jmat),alinvar).tolist()
    var+= alinvar
    goldstone= S(map(lambda x: 'upold({0})'.format(x),range(len(alinvar)))) # goldstone?!
    s= str(dot(alinvar,goldstone)).replace('(','[').replace(')',']')
    icpar= 'norm0'
    s+= ' - ' + icpar
    ics.append(s)
    bifpar.append(icpar)
    bifpar.append(floqexp)
    #initpardict[icpar]= 0
  u2n= dict(zip(arange(len(var))+1,map(str,var)))
  n2u= dict(map(lambda x:x[::-1],u2n.items()))
  if bc=='periodic':
    bc= [v+'0-'+v+'1' for v in map(str,var)]
  if phasecond:
    phaseconstr= str(rhs[n2u[phasecond]-1])
    for s in map(str,var):
      phaseconstr= re.sub(r'\b{0}\b'.format(s),'{0}0'.format(s),phaseconstr)
    bc.append(phaseconstr)
  p2n= dict(zip(arange(len(bifpar))+1,map(str,bifpar)))
  if p2n.has_key(11):
    p2n[max(p2n.keys())+1]= p2n.pop(11)
  p2n[11]= 'period'
  bifpar.append('period')
  n2p= dict(map(lambda x:x[::-1],p2n.items()))
  # write to files
  headername= cfgname.replace('.cfg','BVP.h')
  initpardic= dict(zip(p2n.keys(),zeros(len(p2n))))
  initpardic.update(dict(zip(map(lambda x: n2p[x],bifpar),initbifpar)))
  s= '#include <stdio.h>\n#include <math.h>\n#include "auto_f2c.h"\n'
  s+= '#include "{0}"\n\n'.format(headername)
  s+= join([list2cfunc(list2cf(rhs),bvp=bvp),list2cstpnt(initpar=initpardic),list2cicnd(ics),list2cbcnd(bc),list2cfopt([]),list2cplvs([])],'\n')
  f= open(headername.replace('.h','.c'),'w')
  f.write(s)
  f.close()
  f= open(headername,'w')
  f.write(list2cheader(u2n,p2n))
  f.close()
  d= parseC(os.path.expanduser('~/lib/pylib/c.bvp'))
  d['NINT']= len(ics)
  d['NBC']= len(bc)
  d['ICP']= bifpar
  d['NDIM']= len(rhs)
  d['ICP']= bifpar
  print 'For boundary value problems AUTO requires:'
  print '{0}>={4}={1}+{2}-{3}+1 (ICP>=NBC+NINT-NDIM+1)\n'.format(len(d['ICP']),\
                  d['NBC'],d['NINT'],d['NDIM'],d['NBC']+d['NINT']-d['NDIM']+1)
  f= open('c.'+headername.replace('.h',''),'w')
  f.write(str(d))
  f.close()
  return u2n, p2n

def list2cheader(un={},pn={}):
  from string import join
  un=dict([(x-1,y) for x,y in un.items()])
  pn=dict([(x-1,y) for x,y in pn.items()])
  return join(map(lambda x: '#define {1}\tu[{0}]'.format(*x),un.items())\
            + map(lambda x: '#define {1}\tpar[{0}]'.format(*x),pn.items())\
            + map(lambda x: '#define {1}0\tu0[{0}]'.format(*x),un.items())\
            + map(lambda x: '#define {1}1\tu1[{0}]'.format(*x),un.items()),'\n')
  

def list2cfopt(opt):
  return 'int fopt (integer ndim, const double *u, const integer *icp, const double *par, integer ijac, double *fs, double *dfdu, double *dfdp) {\n  return 0;\n}\n'

def list2cplvs(pls):
  return 'int pvls (integer ndim, const double *u, double *par) {\n  return 0;\n}\n'

def list2cicnd(ic=[]):
  """ """
  from sympy import S
  from sympy.printing import ccode
  from string import join
  ic= map(ccode,(map(S,ic)))
  ic= map(lambda x: '  fi[{0}]= {1};'.format(*x),zip(range(len(ic)),ic))
  s= join(ic,'\n')
  funcs= 'int icnd (integer ndim, const double *par, const integer *icp, integer nint, const double *u, const double *uold, const double *udot, const double *upold, integer ijac, double *fi, double *dint) {{\n{0}\n  return 0;\n}}\n'
  return funcs.format(s)

def list2cbcnd(bc=[]):
  """ """
  from sympy import S
  from sympy.printing import ccode
  from string import join
  bc= map(ccode,(map(S,bc)))
  bc= map(lambda x: '  fb[{0}]= {1};'.format(*x),zip(range(len(bc)),bc))
  s= join(bc,'\n')
  funcs='int bcnd (integer ndim, const double *par, const integer *icp, integer nbc, const double *u0, const double *u1, integer ijac, double *fb, double *dbc) {{\n{0}\n  return 0;\n}}\n'
  return funcs.format(s)

def list2cstpnt(initvar={},initpar={}):
  """ """
  from sympy import S
  from sympy.printing import ccode
  from string import join
  print 'initvar',initvar
  print 'initpar', initpar
  print type(initvar.keys()[0]),type(initpar.keys()[0])
  initvar=dict([(x-1,y) for x,y in initvar.items()])
  initpar=dict([(x-1,y) for x,y in initpar.items()])
  ipar= map(lambda x: '  par[{0}]= {1};'.format(*x),initpar.items())
  iu= map(lambda x: '  u[{0}]= {1};'.format(*x),initvar.items())
  s= join(ipar+iu,'\n')
  funcs='int stpnt (integer ndim, double t, double *u, double *par) {{\n{0}\n  return 0;\n}}\n'
  return funcs.format(s)

def list2cf(rhs):
  """ """
  from sympy import S
  from sympy.printing import ccode
  rhs= map(ccode,(map(S,rhs)))
  r= range(len(rhs))
  return map(lambda x: '  f[{0}]= {1};'.format(*x),zip(r,rhs))
  
  
def list2cfunc(rhs,bvp=True):
  """ """
  from sympy import S
  from sympy.printing import ccode
  from string import join
  if bvp:
    mulper= '  int i;\n  for (i = 0; i < {0}; i++)\n    f[i]*= period;'.format(str(len(rhs)))
  else:
    mulper= ''
  rhs= join(rhs,'\n')
  funcs='int func (integer ndim, const double *u, const integer *icp, const double *par, integer ijac, double *f, double *dfdu, double *dfdp) {{\n{0}\n{1}\n  return 0;\n}}\n'
  return funcs.format(rhs,mulper)

def writecheader(fname,var=[],par=[]):
  """ convert variable names and parameters to c """
  fid= open(fname,'wb')
  fid.writelines('// Variables\n')
  if isinstance(var,dict):
    pass
  else:
    r= range(len(var))
    fid.writelines(map(lambda x: '#define {0} u[{1}]\n'.format(*x),zip(var,r)))
  fid.writelines('\n\n// Parameters\n')
  if isinstance(par,dict):
    pass
  else:
    r= range(len(par))
    if 10 in r:
      r.remove(10)
      r.append(max(r)+1)
    fid.writelines(map(lambda x: '#define {0} par[{1}]\n'.format(*x),zip(par,r)))
  fid.close()

def intoAUTOeqfile(fname,string,rwopt='a'):
  """ print into the auto problem file """
  fid=open(fname,rwopt)
  if ".c" in fname:
    fid.write(string)
  elif ".f90" in fname:
    pass
  else:
    print "No valid file name."
  fid.close()

def diffT(t,f):
  from scipy import gradient, c_, r_, zeros
  b = zeros(f.shape)
  dt = r_[t[1]-t[0],.5*(t[2:]-t[:-2]),t[-1]-t[-2]]
  for k in range(f.shape[1]):
    b[:,k] = gradient(f[:,k],dt)
  return b


def getFloqMult(bifdg, lab):
  """ Extracts Floquet multiplieres from the diagnostics file 
      associated with bifdg.
  """
  import auto
  idx2lab = [k['LAB'] for k in bifdg[0]]
  searchstr = ' ' + str(idx2lab.index(int(lab))) + ' '
  a = capture(auto.fl, bifdg)[1]
  ix = a.find(str(searchstr))
  getF = lambda s: float(s.split()[s.split().index('Multiplier')+2]) + 1j*float(s.split()[s.split().index('Multiplier')+3]) 
  return map(getF, a[ix:].split('\n')[1:1+bifdg(int(lab))['NDIM']])


def get_floqs(bifdg):
  """ Extracts Floquet multiplieres from the diagnostics file 
      associated with bifdg.
  """
  import auto
  a= capture(auto.fl, bifdg)[1]
  b= [x.split() for x in a.split('\n') if ' Multiplier ' in x]
  return [[int(x[1]),float(x[4])+1j*float(x[5])] for x in b]

def capture(func, *args, **kwargs):
  """ Capture the output of func when called with the given arguments.
      The function output includes any exception raised. capture returns
      a tuple of (function result, standard output, standard error).
      Usefull to obtain date that AUTO just prints to STDOUT (like
      Floquet multipliers).
  """
  import sys, cStringIO, traceback
  stdout, stderr = sys.stdout, sys.stderr
  sys.stdout = c1 = cStringIO.StringIO()
  sys.stderr = c2 = cStringIO.StringIO()
  result = None
  try:
    result = func(*args, **kwargs)
  except:
    traceback.print_exc()
  sys.stdout = stdout
  sys.stderr = stderr
  return (result, c1.getvalue(), c2.getvalue())



